@@ -1,7 +1,33 @@
+# file: abdbda.py
+# This file is part of Mother: http://dbmother.org
+#
+# Copyright (c) 2007, Federico Tomassini aka efphe (effetom AT gmail DOT com)
+# All rights reserved.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#
+#     * Redistributions of source code must retain the above copyright
+#       notice, this list of conditions and the following disclaimer.
+#     * Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
+#     * Neither the name of the University of California, Berkeley nor the
+#       names of its contributors may be used to endorse or promote products
+#       derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
+# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
+# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 
-from speaker import Speaker
-from commons import OKI_COL, INF_COL, ERR_COL
-from eccez import QueryError, ConnectionError, BrokenConnection, InvalidFilter
+from mother.speaker import Speaker
+from mother.commons import OKI_COL, INF_COL, ERR_COL
+from mother.eccez import QueryError, ConnectionError, BrokenConnection, InvalidFilter
 
 """
 The Mother DB engine.
@@ -314,10 +340,10 @@ class DbOne(Speaker):
 
         # logging info...
         mogrify= getattr(DbOne._iface_instance, '_mogrify', None)
-        if mogrify is not None:
+        try:
             Speaker.log_info("QSQL- %s", mogrify(s, d))
-        else:
-            Speaker.log_info("QSQL- %s, Filter= %s" % (s, d))
+        except:
+            Speaker.log_info("QSQL- %s, Filter= %s", s, d)
 
         return DbOne._safe_execute(execattr, s, d)
 
@@ -397,6 +423,10 @@ class DbOne(Speaker):
 
         return res
 
+    @staticmethod
+    def close():
+        DbOne._iface_instance._close()
+
 class DbFly(Speaker):
 
     # Iface methods
@@ -549,9 +579,9 @@ class DbFly(Speaker):
                     ERR_COL(type(filter)))
 
         mogrify= getattr(self._iface_instance, '_mogrify', None)
-        if mogrify is not None:
+        try:
             self.log_info("%s: QSQL- %s", INF_COL(self.session_name), mogrify(s, d))
-        else:
+        except:
             self.log_info("%s: QSQL- %s, Filter= %s" , INF_COL(self.session_name), s, d)
 
         return self._safe_execute(execattr, s, d)
@@ -618,6 +648,9 @@ class DbFly(Speaker):
         self._queries_n+= len(l) - 1
         return res
 
+    def close(self):
+        self._iface_instance._close()
+
 class MotherPool:
 
     from Queue import Empty as _empty_queue
@@ -632,6 +665,7 @@ class MotherPool:
     _pool_queue= None
     _pool_type= DB_POOL_LIMITED
     _pool_calm= False
+    _pool_added= []
 
     import threading
 
@@ -721,6 +755,7 @@ class MotherPool:
             q= MotherPool._pool_queue
             p._pool_queue= q
             q.put(p)
+            MotherPool._pool_added.append(p)
 
         MotherPool._pool_current+= n
 
@@ -876,6 +911,10 @@ class MotherPool:
         Speaker.log_warning("Connection %s dropped from Pool.", 
                 ERR_COL(dbfly.session_name))
 
+    @staticmethod
+    def close():
+        for c in MotherPool._pool_added:
+            c.close()
 
 
 def init_abdbda(conf, forced= {}):
@@ -889,7 +928,7 @@ def init_abdbda(conf, forced= {}):
     # will be possible to log them
     err= None
     if not isinstance(conf, dict):
-        import speaker
+        import mother.speaker as speaker
         loc= {}
         names_dict= speaker.__dict__.copy()
         names_dict.update(globals())
@@ -904,7 +943,7 @@ def init_abdbda(conf, forced= {}):
     loc.update(forced)
 
     if hasattr(Speaker, '_spkr_initialized'):
-        from speaker import init_speaker
+        from mother.speaker import init_speaker
         init_speaker(loc)
 
     # now that speaker is up, error is printable
@@ -928,10 +967,13 @@ def init_abdbda(conf, forced= {}):
     pool= loc.get('DB_POOL', False)
     db_one= loc.get('DB_PERSISTENT_ONE', False)
 
+    import atexit
+
     if not pool or db_one:
         DbOne._import_iface()
         #DbOne._connect()
         DbOne._db_initialized= True
+        atexit.register(DbOne.close)
 
     if not pool:
         _DbInfo.db_initialized= True
@@ -958,6 +1000,8 @@ def init_abdbda(conf, forced= {}):
     MotherPool._pool_calm= pool_calm
 
     MotherPool._init_pool()
+    atexit.register(MotherPool.close)
+
     _DbInfo.db_initialized= True
 
     return
