@@ -1,3 +1,31 @@
+# file: mothers.py
+# This file is part of Mother: http://dbmother.org
+#
+# Copyright (c) 2007, Federico Tomassini aka efphe (effetom AT gmail DOT com)
+# All rights reserved.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#
+#     * Redistributions of source code must retain the above copyright
+#       notice, this list of conditions and the following disclaimer.
+#     * Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
+#     * Neither the name of the University of California, Berkeley nor the
+#       names of its contributors may be used to endorse or promote products
+#       derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
+# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
+# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+
+
 #
 ##
 ### MoThEr. Mommas gonna make all of your nightmares come true.
@@ -18,36 +46,23 @@ Where the main Mother Classes and methods are defined:
  * MotherPoolStratus
 """
             
-from commons import ERR_COL, INF_COL, OKI_COL
-from abdbda import DbOne, MoFilter
-from speaker import Speaker
+from mother.speaker import Speaker
 
 #
-##
-### Mother Symbols
-##
+## Mother Symbols
 #
 
-#
-## MoThEr: init flags.
-#
-
-MO_NOA    = 0     # No Action
-MO_DEL    = 1     # Del Action
-MO_UP     = 2     # Update Action
-MO_SAVE   = 3     # Save Action
-MO_LOAD   = 4     # Load Action
-MO_ULOAD  = 5     # Unsafe Load Action
-
-MO_BEFORE = 0     # Before?!?
-MO_AFTER  = 1     # After?!?
+from mother.commons import ERR_COL, INF_COL, OKI_COL
+from mother.commons import MO_NOA, MO_DEL, MO_UP, MO_SAVE, MO_LOAD
+from mother.commons import MO_BEFORE, MO_AFTER
 
 #
 ## MoThEr: SQL symbols.
 #
 
-from abdbda import SQL_DEFAULT, SQL_NULL, SQL_TRUE, SQL_FALSE
-from abdbda import _J, _A
+from mother.abdbda import DbOne, MoFilter
+from mother.abdbda import SQL_DEFAULT, SQL_NULL, SQL_TRUE, SQL_FALSE
+from mother.abdbda import _J, _A
 
 #
 ##
@@ -83,8 +98,8 @@ def init_mother(cfile, fnaming= None):
         return
 
     d= {}
-    import speaker
-    import abdbda
+    import mother.speaker as speaker
+    import mother.abdbda as abdbda
     names_dict= speaker.__dict__.copy()
     names_dict.update(abdbda.__dict__)
     execfile(cfile, names_dict, d)
@@ -127,14 +142,13 @@ def init_mother(cfile, fnaming= None):
             res_str= fnaming('foo')
             assert isinstance(res_str, str)
         except:
-            from speaker import RED
             Speaker.log_warning(
                     "Testing %s. Default fnaming function "
-                    "will be used.", RED('Failed!'))
+                    "will be used.", ERR_COL('Failed!'))
             fnaming= None
     
     if not fnaming:
-        from commons import MotherNaming
+        from mother.commons import MotherNaming
         fnaming= MotherNaming
 
     DbMother.MotherNaming= staticmethod(fnaming)
@@ -157,7 +171,7 @@ def MotherSession(name= None):
     session.endSession(). This call commit the queries and, if the Mother
     Pool is used, put the connection inside the pool. """
 
-    from abdbda import MotherPool
+    from mother.abdbda import MotherPool
 
     if not name:
         import random
@@ -175,7 +189,7 @@ def MotherPoolStatus():
     max: maximum number of connections allowed on the Pool.
     active: number of connections used now."""
 
-    from abdbda import MotherPool
+    from mother.abdbda import MotherPool
 
     if not MotherPool._pool_initialized:
         return ('NoPool', -1, -1, -1, -1, -1)
@@ -185,7 +199,7 @@ def MotherPoolStratus():
     """ MotherPoolStratus() -> string
 
     This is a wrapper of MotherPoolStatus. 
-    The return value is a human readable report."""
+    Differently form MotherPoolStatus, the return value is human readable."""
 
     return """
     Pool Type               ~ %s
@@ -211,7 +225,6 @@ class _DbMap(Speaker):
         if flag == MO_NOA: return _do_nothing
         if flag == MO_SAVE: return obj.insert
         if flag == MO_LOAD: return obj.load
-        if flag == MO_ULOAD: return obj.uload 
         obj.log_int_raise("Invalid flag %s", ERR_COL(flag))
 
     @staticmethod
@@ -498,17 +511,8 @@ class DbMother(_DbMap):
             MO_UP: 'MO_UP', 
             MO_SAVE: 'MO_SAVE', 
             MO_LOAD: 'MO_LOAD', 
-            MO_ULOAD: 'MO_ULOAD'
             }
         
-    @staticmethod
-    def _trigger_actions(obj, flag):
-        if flag == MO_UP: return obj._update
-        if flag == MO_DEL: return obj._delete
-        if flag == MO_SAVE: return obj._insert
-        if flag == MO_LOAD: return obj._load
-        obj.log_int_raise("Invalid flag %s", ERR_COL(flag))
-
     def __init__(self, store= {}, flag= MO_NOA, session= None):
         """ __init__(dict_values, flag, session)
         """
@@ -553,21 +557,17 @@ class DbMother(_DbMap):
         self._initStore(store, flag)
         # ... and enjoy ;)
 
-    def initTriggers(self):
-        """ initTriggers() -> None
-
-        Initialize triggers. Don't call it directly. Use
-        addTrigger instead.\n"""
-
-        self.triggers_map= {}
-        t= self.triggers_map
-            
-        for flag in [MO_UP, MO_LOAD, MO_SAVE, MO_DEL]:
-            t[flag]= {}
-            for when in [MO_AFTER, MO_BEFORE]:
-                t[flag][when]= []
-
-        self._trigger_initialized= True
+    def __str__(self):
+        """ __str__() --> string 
+        """
+        u= INF_COL("Unset")
+        res= "  Table |%s|\n\n" % self.table_name
+        sget= self._store.get
+        for k in self.fields:
+            val= sget(k,u)
+            sep= k in self._moved and '*' or '~'
+            res+= ("  %12s %s %s\n" % (k,sep,str(val)))
+        return res
 
     def _initStore(self, store, flag):
         """ _initStore(store, flag) --> None
@@ -599,27 +599,7 @@ class DbMother(_DbMap):
 
     def unsetParanoia(self):
         setattr(self, 'paranoid', 0)
-        
 
-#
-##
-### Internal Core
-##
-#
-    def _triggers_are_initialized(self):
-        return hasattr(self, 'triggers_map')
-
-    def __str__(self):
-        """ __str__() --> string 
-        """
-        u= INF_COL("Unset")
-        res= "  Table |%s|\n\n" % self.table_name
-        sget= self._store.get
-        for k in self.fields:
-            val= sget(k,u)
-            sep= k in self._moved and '*' or '~'
-            res+= ("  %12s %s %s\n" % (k,sep,str(val)))
-        return res
 
 #
 ##
@@ -841,79 +821,6 @@ class DbMother(_DbMap):
         if not self.session:
             self.commit()
 
-    def get_triggers(self, flag, when):
-        """ get_triggers(flag, when) -> list(functions)
-
-        Returns triggers.
-        """
-
-        if not self._triggers_are_initialized():
-            self.log_int_raise("Trigger are not initialized.")
-
-        t= self.triggers_map
-        try:
-            d= t[flag]
-        except:
-            self.log_int_raise("get_trigger(): invalid flag %s.", ERR_COL(flag))
-
-        try:
-            l= d[when]
-        except:
-            self.log_int_raise("get_trigger(): invalid time %s.", ERR_COL(when))
-
-        return l
-
-    def get_flag_triggers(self, flag):
-        """ get_flag_triggers(flag) -> list(functions)
-
-        Returns triggers.
-        """
-        return  self.get_triggers(flag, MO_BEFORE) + \
-                self.get_triggers(flag, MO_AFTER)
-
-    def has_trigger(self, flag):
-        """ has_trigger(flag) -> bool
-
-        Is there a trigger for the flag `flag`?
-        """
-        if not self._triggers_are_initialized():
-            return False
-
-        return len(self.get_flag_triggers(flag)) > 0
-
-    def add_trigger(self, flag, when, trigger):
-        """ add_trigger(flag, when, f) --> None
-
-        flag= MO_LOAD, MO_SAVE, MO_DEL, MO_UP
-        when= MO_AFTER, MO_BEFORE
-
-        f is the trigger: a python function
-        """
-
-        if not self._triggers_are_initialized():
-            self.initTriggers()
-
-        self.triggers_map[flag][when].append(trigger)
-
-    def trigger(self, flag, when, *args):
-        """ trigger(flag, when, *args) --> None
-
-        flag= MO_LOAD, MO_SAVE, MO_DEL, MO_UP
-        when= MO_AFTER, MO_BEFORE
-
-        execute the tasked triggers for action flag and when
-        """
-
-        if not self._triggers_are_initialized():
-            self.log_int_raise("Triggers are not initialized.")
-
-        l= self.get_triggers(flag, when)
-        when_str= when == MO_AFTER and 'After' or 'Before'
-        for f in l:
-            f(*args)
-            self.log_info("Trigger %s (%s-%s) Fired!", 
-                    OKI_COL(f.func_name), when_str, self._mo_flags[flag])
-
     def setField(self, field_name, field_value):
         """ setField(field_name, field_value) --> None
 
@@ -1044,7 +951,7 @@ class DbMother(_DbMap):
         qry= self._sqlInsert()
         self._moved.clear()
 
-    def _update(self):
+    def _update(self, updict= None):
 
         # Really need to update?
         if not self._moved:
@@ -1065,7 +972,7 @@ class DbMother(_DbMap):
 
         # Delete all informations...  
         self._moved.clear()
-        self._store.clear()
+        #self._store.clear()
 
     def _load(self, fields= None):
 
@@ -1073,7 +980,7 @@ class DbMother(_DbMap):
         mf= fields and set(fields) or set(self._fieldsMissing())
 
         if not mf <= self.fields:
-            from eccez import SelectError
+            from mother.eccez import SelectError
             self.log_raise("Invalid Fields %s for table %s.",
                     ERR_COL(list(mf - self.fields)), 
                     self.table_name, SelectError)
@@ -1090,35 +997,13 @@ class DbMother(_DbMap):
         # Useful return, that's all
         return self.getFields()
 
-    def _triggered_action(self, flag, *args):
-
-        f= self._trigger_actions(self, flag)
-
-        if not self._triggers_are_initialized() or \
-           not self.get_flag_triggers(flag):
-            res= f(*args)
-        else:
-            self._safeBeginTrans()
-            try:
-                self.trigger(flag, MO_BEFORE)
-                res= f(*args)
-                self.trigger(flag, MO_AFTER)
-
-            except Exception, s:
-                self._safeRollback()
-                self.log_int_raise(str(s))
-
-            self._safeCommit()
-
-        self.log_info("Action %s.", OKI_COL("completed"))
-        return res
-
     def delete(self):
         """ delete() --> None
 
         Really do you want to kill me?
         """
-        self._triggered_action(MO_DEL)
+        self._delete()
+        self.log_info("Action %s.", OKI_COL("completed"))
     
     def load(self, fields= None):
         """ load([fields= None]) --> dict
@@ -1127,7 +1012,9 @@ class DbMother(_DbMap):
         if fields is not None, only fields specified will
         be loaded.
         """
-        return self._triggered_action(MO_LOAD,fields)
+        res= self._load(fields)
+        self.log_info("Action %s.", OKI_COL("completed"))
+        return res
 
     def insert(self):
         """ insert() --> None
@@ -1135,7 +1022,8 @@ class DbMother(_DbMap):
         Insert to db the values stored in store.
         For field with no value, SQL_DEFAULT is assumed.
         """
-        self._triggered_action(MO_SAVE)
+        self._insert()
+        self.log_info("Action %s.", OKI_COL("completed"))
         
     def update(self, updict= None):
         """ update([updict= None]) --> None
@@ -1148,16 +1036,8 @@ class DbMother(_DbMap):
         if updict:
             self.setFields(updict)
 
-        self._triggered_action(MO_UP)
-
-    def uload(self, fields= None):
-        """ uload([fields= None]) --> dict
-
-        """
-        self.log_warning(ERR_COL("Obsolete action MO_ULOAD: from Mother "
-                         "0.6.0 MO_ULOAD and MO_LOAD are equivalent. "
-                         "MO_ULOAD will be dropped. Use MO_LOAD."))
-        return self.load(fields)
+        self._update()
+        self.log_info("Action %s.", OKI_COL("completed"))
 
     def refresh(self, d, flag= MO_NOA):
         """ refresh(d[,flag=MO_NOA]) --> None
@@ -1618,11 +1498,40 @@ class MotherManager:
         if flag_obj: return r_obj
         else: return r_obj.getFields()
 
-    def initManyManager(self, children):
+    def initManyManager(self, children= None):
+        """initManyManager(self [, children= None]) --> None
+        
+        Calling this method, a family of function will be created
+        inside your Mother instance: the ManyManager family.
+
+        `children` is a list or None.
+        If None, for each table child the following methods will be created:
+
+            insertMany<Child>
+            deleteMany<Child>
+            updateMany<Child>
+            loadMany<Child>
+
+        `children` can also be a list of table names. For each table
+        the same methods will be created.
+
+        Finally, `children` can be a list of DbMother classes: this is
+        useful, because it's possible to handle children using your
+        custom DbMother classes.
+
+        * NOTE: methods already defined are not overwritten.
+        """
 
         self.log_info("The initManyManager() methods are not tested deeply: if You "\
                       "encounter bugs, don't panic: please, signal them.")
 
+        if not children:
+            children= [getMotherBuilder(c) 
+                    for c in self._table_children(self.table_name)]
+        else:
+            children= [isinstance(c, str) and getMotherBuilder(c) 
+                        or c for c in children]
+
         def handle_children(builder, flag):
             def fly_handle_children(dlist, fields= None):
                 return self.handleManyChildren(builder, flag, dlist, fields)
@@ -1661,32 +1570,38 @@ class MotherManager:
 
               setattr(self, attr_name, attr)
 
-    def initChildManager(self, children):
-        """initChildManager(self,children) --> None
-        
-        You can define a Mother class for each Child-TABLE.
-        In fact, children is a list of Mother Builders.
-        The following functions will be wrappered:
-            
-            insertChild(Foo,dict)
-            deleteChildren(Foo, filter)
-            updateChildren(Foo, dict, filter)
-            getChild(Foo, dict)
-            getChildren(builder,fields,filter,jbuilder,jfilter,outer,order)
+    def initChildManager(self, children= None):
+        """initChildManager(self [, children= None]) --> None
 
+        Calling this method, a family of function will be created
+        inside your Mother instance: the ChildManager family.
 
-        So, if FOO is a child, you can create the Mother Class
-        Foo and have the methods:
+        `children` is a list or None.
+        If None, for each table child the following methods will be created:
 
-            insertFoo(dict)
-            deleteMultipleFoo(filter)
-            updateMultipleFoo(dict, filter)
-            getFoo(dict)
-            getMultipleFoo(fields, filter, jbuilder, jfilter, outer, order)
+            insert<Child>
+            get<Child>
+            deleteMultiple<Child>
+            updateMultiple<Child>
+            getMultiple<Child>
+
+        `children` can also be a list of table names. For each table
+        the same methods will be created.
+
+        Finally, `children` can be a list of DbMother classes: this is
+        useful, because it's possible to handle children using your
+        custom DbMother classes.
 
         * NOTE: methods already defined are not overwritten.
         """
 
+        if not children:
+            children= [getMotherBuilder(c) 
+                    for c in self._table_children(self.table_name)]
+        else:
+            children= [isinstance(c, str) and getMotherBuilder(c) 
+                        or c for c in children]
+
         def insert_child(builder):
             def fly_insert_child(d):
                 return self.insertChild(builder, d)
@@ -1769,12 +1684,10 @@ class MotherManager:
                 attr.__doc__=\
                         " %s(dict) --> DbMother\n\n"                                    \
                         "retrieve a unique child.\n"                                    \
-                        "Here dict acts has a filter. Note that Mother will try to\n"   \
-                        "'MO_ULOAD' the record, so dict does not have to contain\n"     \
-                        "necessarily the primary key for your record.\n"                \
+                        "Here dict acts has a filter and does not have to contain\n"    \
+                        "the primary key for your record.\n"                            \
+                        "If a unique record is not found, an exception will be raised." \
                         "Don't specify the foreign key: it's assigned automatically.\n" \
-                        "This method has the same beahaviour of DbMother.uload(): \n"   \
-                        "if a unique record is not found, an exception will be raised." \
                         % attr_name
 
                 setattr(self, attr_name, attr)
@@ -1793,27 +1706,31 @@ class MotherManager:
                 setattr(self, attr_name, attr)
 
     def initRelationManager(self, rels):
-        """initRelationManager(children) --> None
-        
-        This is a magic function.
-        It's a wrapper for the methods:
 
-            - assigRelation
-            - joinRelation
-            - dropRelation
-            - dropRelations
-            - relParams
+        """initRelationManager(self, rels) --> None
 
-        The wrappers will be:
+        Calling this method, a family of function will be created
+        inside your Mother instance: the RelationManager family.
 
-            - assigFoo
-            - joinFoo
-            - dropFoo
-            - dropMultipleFoo
-            - paramsFoo
+        `rels` is a list of DbMother classes or table names.
+        For each related table, the following methods will be created:
 
-        NOTE: methods already defined are not overwritten.
+            assign<Rel>
+            join<Rel>
+            drop<Rel>
+            dropMultiple<Rel>
+            updateMultiple<Rel>
+            parmas<Rel>
+
+        `children` can be a list of DbMother classes: this is
+        useful, because it's possible to handle related records using your
+        custom DbMother classes.
+
+        * NOTE: methods already defined are not overwritten.
         """
+       
+        rels= [isinstance(c, str) and getMotherBuilder(c) 
+                    or c for c in rels]
 
         def assign_rel(builder):
             def fly_assign_rel(d, flag= MO_NOA, params= {}):
@@ -1897,7 +1814,7 @@ class MotherManager:
             attr_name= camel(rtbl, 'params')
             if not hasattr(self, attr_name):
                 attr= rel_params(rel)
-                attr.__doc__=\
+                attr.__doc__=                                                               \
                     " %s(obj [,fields= None, flag_obj= False]) --> dict Mother Object\n\n"  \
                     "Returns, depending on flag_obj,  a dict or a Mother Object that\n"     \
                     "represents the relation between `self' and the record on the table\n"  \
@@ -1916,17 +1833,19 @@ class MotherManager:
 #
 
 class MotherBox(DbOne):
+
     def __init__(self, builder, filter= None, flag= MO_NOA, 
                     fields= None, order= None, session= None, 
-                    distinct= False, notriggers= False):
+                    distinct= False):
 
         iface= session or DbOne
         iface.export_iface(self)
         self.session= session
 
-        self.builder= builder
+        self.builder= isinstance(builder, str) \
+                and getMotherBuilder(builder) or builder
+
         self.momma= builder(session= session)
-        self.notriggers= notriggers
     
         if filter:
             if not isinstance(filter, MoFilter):
@@ -1953,33 +1872,21 @@ class MotherBox(DbOne):
             return
 
         else:
-            self.log_int_raise("Invalid Flag %s for MotherBox", ERR_COL(flag))
+            self.log_int_raise("Invalid Flag %s for MotherBox", 
+                    ERR_COL(flag))
 
         del self.momma
 
 
     def __len__(self):
+
         return len(self._store)
 
-    def _retrieve_mothers(self, filter):
+    def __iter__(self):
+        for d in self._store:
+            yield d
 
-        where= filter or ''
-        tbl= self.momma.table_name
-        fields= self.momma.pkeys
-        fields= _J([f for f in fields])
-
-        qry= "SELECT %(fields)s from %(tbl)s %(where)s" % locals()
-
-        res= self.mr_query(qry)
-        session= self.session
-        mothers= [self.builder(d, session= session) for d in res]
-        return mothers
-
-    def _trigger_mothers(self, mothers, flag, when):
-        for m in mothers:
-            m.trigger(flag, when)
-
-    def _loadBox(self, filter, fields, order, distinct):
+    def loadBox(self, filter, fields, order, distinct):
 
         tbl= self.builder.table_name
 
@@ -2009,31 +1916,7 @@ class MotherBox(DbOne):
         self.log_info("Action %s.", OKI_COL("completed"))
 
 
-    def loadBox(self, filter, fields, order, distinct):
-
-        if self.notriggers or not self.momma.has_trigger(MO_LOAD):
-            self._loadBox(filter, fields, order, distinct)
-            return 
-            
-        self.log_info("Launching Triggers on Box (MO_LOAD)...")
-        ms= self._retrieve_mothers(filter)
-        self._trigger_mothers(ms, MO_LOAD, MO_BEFORE)
-        self._loadBox(filter, fields, order, distinct)
-        self._trigger_mothers(ms, MO_LOAD, MO_AFTER)
-
-    def updateBox(self, d, filter= None):
-
-        if self.notriggers or not self.momma.has_trigger(MO_UP):
-            self._updateBox(d, filter)
-            return
-
-        self.log_info("Launching Triggers on Box (MO_UP) operation...")
-        ms= self._retrieve_mothers(filter)
-        self._trigger_mothers(ms, MO_UP, MO_BEFORE)
-        self._updateBox(d, filter)
-        self._trigger_mothers(ms, MO_UP, MO_AFTER)
-
-    def _updateBox(self, filter, d):
+    def updateBox(self, filter, d):
         """ updateBox(self, filter, d): --> None
 
         Update a set of records.
@@ -2076,18 +1959,6 @@ class MotherBox(DbOne):
         self.log_info("Action %s.", OKI_COL("completed"))
 
     def deleteBox(self, filter=None):
-
-        if self.notriggers or not self.momma.has_trigger(MO_DEL):
-            self._deleteBox(filter)
-            return 
-        
-        self.log_info("Launching Triggers on Box (MO_DEL) operations...")
-        ms= self._retrieve_mothers(filter) 
-        self._trigger_mothers(ms, MO_DEL, MO_BEFORE)
-        self._deleteBox(filter)
-        self._trigger_mothers(ms, MO_DEL, MO_AFTER)
-
-    def _deleteBox(self, filter=None):
         """ deleteBox(self, filter= None, stored= False) --> None
 
         delete from DB.
@@ -2115,16 +1986,6 @@ class MotherBox(DbOne):
         else:
             return self._store
 
-    def getFields(self, flag_obj= False):
-        """ getRecords(self, flag_obj= False) -> list(dict) or list(Mothers)
-
-        The return value depends on flag_obj.
-        MotherBox.getFields is obsolete! Use MotherBox.getRecords!
-        """
-        self.log_warning(ERR_COL(
-            "Obsolete Method MotherBox.getFields(). Use getRecords() instead! "
-            "This method will be removed from Mother."))
-        return self.getRecords(flag_obj)
 
 #
 ##
@@ -2180,6 +2041,10 @@ class MotherFusion(_DbMap):
             self.rtbl= rtbl
             self.joinBuilders()
 
+    def __iter__(self):
+        for d in self._store:
+            yield d
+
     def swap(self, a, b, fields):
 
         ta= a.table_name
@@ -2374,6 +2239,11 @@ class MotherMany(_DbMap):
 
         return len(self._records)
 
+    def __iter__(self):
+
+        for d in self.store:
+            yield d
+
     def addRows(self, rows):
 
         try:
